CheckSupport

Struct CheckSupport 

pub struct CheckSupport<'a> { /* private fields */ }
Request to check if a server advertises support for a specific protocol extension.

Sends an OPTIONS request to the specified URI and checks if the DAV: header contains the expected value.

Example

let uri = Uri::from_static("/");
let response = webdav.request(
    CheckSupport::new(&uri, "calendar-access")
).await?;

Implementations

Source

impl<'a> CheckSupport<'a>

pub fn new(uri: &'a Uri, expectation: &'a str) -> Self

Create a new CheckSupport request.

uri is the URI to check. expectation is the value that must be present in the DAV header.